@@ -16,7 +16,7 @@ class CmdExport(base.BaseCommand):
 
         dst_rev = self.args.dst_rev
         if not dst_rev:
-            task_id = f"exporting-task-{int(time.time())}"
+            task_id = f"exporting-task-{float(time.time())}"
             dst_rev = f"{task_id}@{task_id}"
 
         return CmdExport.run_with_args(mir_root=self.args.mir_root,
@@ -44,9 +44,8 @@ class CmdExport(base.BaseCommand):
             logging.error(f"invalid --format: {format}")
             return MirCode.RC_CMD_INVALID_ARGS
 
-        src_typ_rev_tid = revs_parser.parse_single_arg_rev(src_revs)
-        if checker.check_src_revs(src_typ_rev_tid) != MirCode.RC_OK:
-            return MirCode.RC_CMD_INVALID_ARGS
+        src_rev_tid = revs_parser.parse_single_arg_rev(src_revs, need_tid=False)
+        dst_rev_tid = revs_parser.parse_single_arg_rev(dst_rev, need_tid=True)
 
         PhaseLoggerCenter.create_phase_loggers(top_phase='export',
                                                monitor_file=mir_repo_utils.work_dir_to_monitor_file(work_dir),
@@ -59,10 +58,8 @@ class CmdExport(base.BaseCommand):
         format_type = data_exporter.format_type_from_str(format)
 
         # asset ids
-        mir_metadatas: mirpb.MirMetadatas = mir_storage_ops.MirStorageOps.load_single(mir_root=mir_root,
-                                                                                      mir_branch=src_typ_rev_tid.rev,
-                                                                                      mir_task_id=src_typ_rev_tid.tid,
-                                                                                      ms=mirpb.MIR_METADATAS)
+        mir_metadatas: mirpb.MirMetadatas = mir_storage_ops.MirStorageOps.load_single_storage(
+            mir_root=mir_root, mir_branch=src_rev_tid.rev, mir_task_id=src_rev_tid.tid, ms=mirpb.MIR_METADATAS)
         asset_ids = set()
         for k in mir_metadatas.attributes.keys():
             asset_ids.add(str(k))
@@ -76,47 +73,31 @@ class CmdExport(base.BaseCommand):
         # export
         data_exporter.export(mir_root=mir_root,
                              assets_location=media_location,
-                             class_type_ids={type_id: type_id for type_id in type_ids_list},
+                             class_type_ids={type_id: type_id
+                                             for type_id in type_ids_list},
                              asset_ids=asset_ids,
                              asset_dir=asset_dir,
                              annotation_dir=annotation_dir,
                              need_ext=True,
                              need_id_sub_folder=False,
-                             base_branch=src_typ_rev_tid.rev,
-                             base_task_id=src_typ_rev_tid.tid,
+                             base_branch=src_rev_tid.rev,
+                             base_task_id=src_rev_tid.tid,
                              format_type=format_type)
 
         # add task result commit
-        CmdExport._commit(mir_root=mir_root, src_revs=src_revs, dst_rev=dst_rev)
+        task = mir_storage_ops.create_task(task_type=mirpb.TaskType.TaskTypeExportData,
+                                           task_id=dst_rev_tid.tid,
+                                           message=f"export from {src_rev_tid.rev_tid}")
+        mir_storage_ops.MirStorageOps.save_and_commit(mir_root=mir_root,
+                                                      mir_branch=dst_rev_tid.rev,
+                                                      his_branch=src_rev_tid.rev,
+                                                      mir_datas={},
+                                                      task=task)
 
         return MirCode.RC_OK
 
-    @staticmethod
-    def _commit(mir_root: str, src_revs: str, dst_rev: str) -> None:
-        src_rev_tid = revs_parser.parse_single_arg_rev(src_revs)
-        dst_rev_tid = revs_parser.parse_single_arg_rev(dst_rev)
 
-        task = mirpb.Task()
-        task.return_code = MirCode.RC_OK
-        task.type = mirpb.TaskTypeExportData
-        task.task_id = dst_rev_tid.tid
-        task.timestamp = int(time.time())
-        task.name = dst_rev_tid.tid
-
-        mir_tasks = mirpb.MirTasks()
-        mir_tasks.head_task_id = dst_rev_tid.tid
-        mir_tasks.tasks[dst_rev_tid.tid].CopyFrom(task)
-
-        mir_storage_ops.MirStorageOps.save_and_commit(mir_root=mir_root,
-                                                      mir_branch=dst_rev_tid.rev,
-                                                      task_id=dst_rev_tid.tid,
-                                                      his_branch=src_rev_tid.rev,
-                                                      mir_datas={mirpb.MIR_TASKS: mir_tasks},
-                                                      commit_message=task.name)
-
-
-def bind_to_subparsers(subparsers: argparse._SubParsersAction,
-                       parent_parser: argparse.ArgumentParser) -> None:
+def bind_to_subparsers(subparsers: argparse._SubParsersAction, parent_parser: argparse.ArgumentParser) -> None:
     exporting_arg_parser = subparsers.add_parser('export',
                                                  parents=[parent_parser],
                                                  description='use this command to export data',
@@ -148,7 +129,8 @@ def bind_to_subparsers(subparsers: argparse._SubParsersAction,
                                       default="none",
                                       choices=data_exporter.support_format_type(),
                                       help='annotation format: ark / voc / none')
-    exporting_arg_parser.add_argument("-p", '--cis',
+    exporting_arg_parser.add_argument("-p",
+                                      '--cis',
                                       dest="in_cis",
                                       type=str,
                                       required=False,
