@@ -1,15 +1,14 @@
 import argparse
-import datetime
 import logging
 from typing import Any, Callable, List, Tuple, Optional, Set, Union
 
 from mir.commands import base
 from mir.protos import mir_command_pb2 as mirpb
 from mir.tools import checker, class_ids, mir_repo_utils, mir_storage, mir_storage_ops, revs_parser
+from mir.tools.code import MirCode
 from mir.tools.command_run_in_out import command_run_in_out
 from mir.tools.errors import MirRuntimeError
 from mir.tools.phase_logger import PhaseLoggerCenter
-from mir.tools.code import MirCode
 
 # type for function `__include_match` and `__exclude_match`
 __IncludeExcludeCallableType = Callable[[Set[str], mirpb.MirKeywords, str, Any], Set[str]]
@@ -71,23 +70,8 @@ class CmdFilter(base.BaseCommand):
         in_cks = in_cks.strip() if in_cks else ''
         ex_cks = ex_cks.strip() if ex_cks else ''
 
-        if not in_cis and not ex_cis and not in_cks and not ex_cks:
-            logging.error("invalid args: empty -p, -P, -c and -C")
-            return MirCode.RC_CMD_INVALID_ARGS
-
-        if not src_revs:
-            logging.error("invalid args: empty --src-revs")
-            return MirCode.RC_CMD_INVALID_ARGS
-        src_typ_rev_tid = revs_parser.parse_single_arg_rev(src_revs)
-        if checker.check_src_revs(src_typ_rev_tid) != MirCode.RC_OK:
-            return MirCode.RC_CMD_INVALID_ARGS
-
-        if not dst_rev:
-            logging.error("invalid args: --dst-rev")
-            return MirCode.RC_CMD_INVALID_ARGS
-        dst_typ_rev_tid = revs_parser.parse_single_arg_rev(dst_rev)
-        if checker.check_dst_rev(dst_typ_rev_tid) != MirCode.RC_OK:
-            return MirCode.RC_CMD_INVALID_ARGS
+        src_typ_rev_tid = revs_parser.parse_single_arg_rev(src_revs, need_tid=False)
+        dst_typ_rev_tid = revs_parser.parse_single_arg_rev(dst_rev, need_tid=True)
 
         PhaseLoggerCenter.create_phase_loggers(top_phase='filter',
                                                monitor_file=mir_repo_utils.work_dir_to_monitor_file(work_dir),
@@ -98,14 +82,12 @@ class CmdFilter(base.BaseCommand):
         if return_code != MirCode.RC_OK:
             return return_code
 
-        mir_contents = mir_storage_ops.MirStorageOps.load(mir_root=mir_root,
-                                                          mir_branch=src_typ_rev_tid.rev,
-                                                          mir_task_id=src_typ_rev_tid.tid,
-                                                          mir_storages=mir_storage.get_all_mir_storage())
-        mir_metadatas: mirpb.MirMetadatas = mir_contents[mirpb.MirStorage.MIR_METADATAS]
-        mir_annotations: mirpb.MirAnnotations = mir_contents[mirpb.MirStorage.MIR_ANNOTATIONS]
-        mir_keywords: mirpb.MirKeywords = mir_contents[mirpb.MirStorage.MIR_KEYWORDS]
-        mir_tasks: mirpb.MirTasks = mir_contents[mirpb.MirStorage.MIR_TASKS]
+        [mir_metadatas, mir_annotations, mir_keywords, mir_tasks,
+         _] = mir_storage_ops.MirStorageOps.load_multiple_storages(mir_root=mir_root,
+                                                                   mir_branch=src_typ_rev_tid.rev,
+                                                                   mir_task_id=src_typ_rev_tid.tid,
+                                                                   ms_list=mir_storage.get_all_mir_storage(),
+                                                                   as_dict=False)
         task_id = dst_typ_rev_tid.tid
         base_task_id = mir_annotations.head_task_id
 
@@ -140,11 +122,6 @@ class CmdFilter(base.BaseCommand):
                 asset_ids_set = match_func(asset_ids_set, mir_keywords, attr_name, ci_ck_conditions)
                 logging.info(f"assets count after {message}: {len(asset_ids_set)}")
 
-        if not asset_ids_set:
-            raise MirRuntimeError(
-                error_code=MirCode.RC_CMD_INVALID_ARGS,
-                error_message=f"matched nothing with pred: {in_cis}, excludes: {ex_cis}, please try another preds")
-
         matched_mir_metadatas = mirpb.MirMetadatas()
         matched_mir_annotations = mirpb.MirAnnotations()
         matched_mir_keywords = mirpb.MirKeywords()
@@ -166,31 +143,26 @@ class CmdFilter(base.BaseCommand):
             matched_mir_annotations.task_annotations[task_id].image_annotations[asset_id].CopyFrom(
                 base_task_annotations.image_annotations[asset_id])
 
-        # generate matched tasks
-        task = mirpb.Task()
-        task.type = mirpb.TaskType.TaskTypeFilter
-        task.name = f"filter bid: {base_task_id}, tid: {task_id}, select: {in_cis} exclude: {ex_cis}"
-        task.base_task_id = base_task_id
-        task.task_id = task_id
-        task.timestamp = int(datetime.datetime.now().timestamp())
-        mir_storage_ops.add_mir_task(mir_tasks, task)
-
         logging.info("matched: %d, overriding current mir repo", len(matched_mir_metadatas.attributes))
 
         PhaseLoggerCenter.update_phase(phase='filter.change')
 
+        commit_message = f"filter select: {in_cis} exclude: {ex_cis}"
+        task = mir_storage_ops.create_task(task_type=mirpb.TaskType.TaskTypeFilter,
+                                           task_id=task_id,
+                                           message=commit_message,
+                                           src_revs=src_revs,
+                                           dst_rev=dst_rev)
         matched_mir_contents = {
             mirpb.MirStorage.MIR_METADATAS: matched_mir_metadatas,
             mirpb.MirStorage.MIR_ANNOTATIONS: matched_mir_annotations,
-            mirpb.MirStorage.MIR_TASKS: mir_tasks,
         }
 
         mir_storage_ops.MirStorageOps.save_and_commit(mir_root=mir_root,
                                                       mir_branch=dst_typ_rev_tid.rev,
-                                                      task_id=task_id,
                                                       his_branch=src_typ_rev_tid.rev,
                                                       mir_datas=matched_mir_contents,
-                                                      commit_message=task.name)
+                                                      task=task)
 
         return MirCode.RC_OK
 
@@ -206,8 +178,7 @@ class CmdFilter(base.BaseCommand):
                                        work_dir=self.args.work_dir)
 
 
-def bind_to_subparsers(subparsers: argparse._SubParsersAction,
-                       parent_parser: argparse.ArgumentParser) -> None:
+def bind_to_subparsers(subparsers: argparse._SubParsersAction, parent_parser: argparse.ArgumentParser) -> None:
     filter_arg_parser = subparsers.add_parser("filter",
                                               parents=[parent_parser],
                                               description="use this command to filter assets",
