@@ -1,11 +1,8 @@
 import argparse
-import datetime
 import logging
 import os
-import shutil
 import subprocess
 from subprocess import CalledProcessError
-import tarfile
 import traceback
 from typing import Any, List, Optional, Set, Tuple
 
@@ -13,8 +10,8 @@ import yaml
 
 from mir.commands import base
 from mir.protos import mir_command_pb2 as mirpb
-from mir.tools import checker, class_ids, data_exporter, hash_utils, mir_storage_ops, revs_parser
-from mir.tools import utils as mir_utils
+from mir.tools import checker, class_ids, context, data_exporter, mir_storage_ops, revs_parser
+from mir.tools import settings as mir_settings, utils as mir_utils
 from mir.tools.command_run_in_out import command_run_in_out
 from mir.tools.code import MirCode
 from mir.tools.errors import MirRuntimeError
@@ -23,21 +20,25 @@ from mir.tools.errors import MirRuntimeError
 # private: post process
 def _process_model_storage(out_root: str, model_upload_location: str, executor_config: dict,
                            task_context: dict) -> Tuple[str, float]:
-    model_paths, model_mAP = _find_models(os.path.join(out_root, "models"))
+    """
+    find and save models
+    Returns:
+        model hash, model mAP and ModelStorage
+    """
+    out_model_dir = os.path.join(out_root, "models")
+    model_paths, model_mAP = _find_models(out_model_dir)
     if not model_paths:
         # if have no models
         return '', model_mAP
 
-    tar_path = os.path.join(out_root, "models.tar.gz")
-    _pack_models_and_config(model_paths=model_paths,
-                            executor_config=executor_config,
-                            task_context=dict(**task_context, mAP=model_mAP, type='training'),
-                            dest_path=tar_path)
-    model_sha1 = hash_utils.sha1sum_for_file(tar_path)
-
-    dest_path = os.path.join(model_upload_location, model_sha1)
-    _upload_model_pack(tar_path, dest_path)
-    os.remove(tar_path)
+    model_storage = mir_utils.ModelStorage(executor_config=executor_config,
+                                           task_context=dict(**task_context,
+                                                             mAP=model_mAP,
+                                                             type=mirpb.TaskType.TaskTypeTraining),
+                                           models=[os.path.basename(model_path) for model_path in model_paths])
+    model_sha1 = mir_utils.pack_and_copy_models(model_storage=model_storage,
+                                                model_dir_path=out_model_dir,
+                                                model_location=model_upload_location)
 
     return model_sha1, model_mAP
 
@@ -65,66 +66,7 @@ def _find_models(model_root: str) -> Tuple[List[str], float]:
         logging.warning(traceback.format_exc())
         return [], 0.0
 
-    return ([os.path.join(model_root, name) for name in model_names], model_mAP)
-
-
-def _pack_models_and_config(model_paths: List[str], executor_config: dict, task_context: dict, dest_path: str) -> bool:
-    if not model_paths or not dest_path:
-        raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
-                              error_message='invalid model_paths or dest_path')
-
-    logging.info(f"packing models to {dest_path}")
-    model_storage = mir_utils.ModelStorage(executor_config=executor_config,
-                                           task_context=task_context,
-                                           models=[os.path.basename(model_path) for model_path in model_paths])
-    ymir_info_file_name = 'ymir-info.yaml'
-    ymir_info_file_path = os.path.join(os.path.dirname(dest_path), ymir_info_file_name)
-    with open(ymir_info_file_path, 'w') as f:
-        f.write(yaml.dump(model_storage.as_dict()))
-
-    with tarfile.open(dest_path, "w:gz") as dest_tar_gz:
-        for model_path in model_paths:
-            logging.info(f"    packing {model_path} -> {os.path.basename(model_path)}")
-            dest_tar_gz.add(model_path, os.path.basename(model_path))
-
-        # pack ymir-info.yaml
-        logging.info(f"    packing {ymir_info_file_path} -> {ymir_info_file_name}")
-        dest_tar_gz.add(ymir_info_file_path, ymir_info_file_name)
-    return True
-
-
-def _upload_model_pack(model_pack_path: str, dest_path: str) -> bool:
-    if not model_pack_path or not dest_path:
-        raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
-                              error_message='invalid model_pack_path or dest_path')
-
-    shutil.copyfile(model_pack_path, dest_path)
-    return True
-
-
-def _update_mir_tasks(mir_root: str, src_rev_tid: revs_parser.TypRevTid, dst_rev_tid: revs_parser.TypRevTid,
-                      model_sha1: str, mAP: float, task_ret_code: int, task_err_msg: str) -> mirpb.MirTasks:
-    """
-    add a new mir single task into mir_tasks from branch base_branch, and save it to a new branch: dst_branch
-    """
-    logging.info("creating task id: {}, model hash: {}, mAP: {}".format(dst_rev_tid.tid, model_sha1, mAP))
-
-    task = mirpb.Task()
-    task.type = mirpb.TaskTypeTraining
-    task.name = "training done"
-    task.task_id = dst_rev_tid.tid
-    task.timestamp = int(datetime.datetime.now().timestamp())
-    task.model.model_hash = model_sha1
-    task.model.mean_average_precision = mAP
-    task.return_code = task_ret_code
-    task.return_msg = task_err_msg
-
-    mir_tasks: mirpb.MirTasks = mir_storage_ops.MirStorageOps.load_single(mir_root=mir_root,
-                                                                          mir_branch=src_rev_tid.rev,
-                                                                          mir_task_id=src_rev_tid.tid,
-                                                                          ms=mirpb.MirStorage.MIR_TASKS)
-    mir_storage_ops.add_mir_task(mir_tasks, task)
-    return mir_tasks
+    return ([os.path.join(model_root, os.path.basename(name)) for name in model_names], model_mAP)
 
 
 # private: process
@@ -152,30 +94,26 @@ def _run_train_cmd(cmd: List[str], out_log_path: str) -> int:
 
 
 # private: pre process
-def _generate_config(config: Any, out_config_path: str, task_id: str, pretrained_model_params: List[str]) -> dict:
-    config["task_id"] = task_id
+def _generate_config(executor_config: Any, out_config_path: str, task_id: str,
+                     pretrained_model_params: List[str]) -> dict:
+    executor_config["task_id"] = task_id
     if pretrained_model_params:
-        config['pretrained_model_params'] = pretrained_model_params
-    elif 'pretrained_model_params' in config:
-        del config['pretrained_model_params']
+        executor_config['pretrained_model_params'] = pretrained_model_params
+    elif 'pretrained_model_params' in executor_config:
+        del executor_config['pretrained_model_params']
 
-    container_config = config.copy()
-    container_config['gpu_id'] = mir_utils.map_gpus_zero_index(config.get('gpu_id', ''))
-
-    logging.info("container config: {}".format(container_config))
+    logging.info("container config: {}".format(executor_config))
 
     with open(out_config_path, "w") as f:
-        yaml.dump(container_config, f)
+        yaml.dump(executor_config, f)
 
-    return config
+    return executor_config
 
 
-def _get_shm_size(config_file_path: str) -> str:
-    with open(config_file_path, 'r') as f:
-        config = yaml.safe_load(f.read())
-    if 'shm_size' not in config:
+def _get_shm_size(executor_config: dict) -> str:
+    if 'shm_size' not in executor_config:
         return '16G'
-    return config['shm_size']
+    return executor_config['shm_size']
 
 
 def _prepare_pretrained_models(model_location: str, model_hash: str, dst_model_dir: str,
@@ -210,11 +148,16 @@ def _prepare_pretrained_models(model_location: str, model_hash: str, dst_model_d
     return model_storage.models
 
 
+def _get_task_parameters(config: dict) -> str:
+    return config.get(mir_settings.TASK_CONTEXT_KEY, {}).get(mir_settings.TASK_CONTEXT_PARAMETERS_KEY, '')
+
+
 class CmdTrain(base.BaseCommand):
     def run(self) -> int:
         logging.debug("command train: %s", self.args)
 
         return CmdTrain.run_with_args(work_dir=self.args.work_dir,
+                                      asset_cache_dir=self.args.asset_cache_dir,
                                       model_upload_location=self.args.model_path,
                                       pretrained_model_hash=self.args.model_hash,
                                       src_revs=self.args.src_revs,
@@ -223,69 +166,71 @@ class CmdTrain(base.BaseCommand):
                                       media_location=self.args.media_location,
                                       tensorboard_dir=self.args.tensorboard_dir,
                                       executor=self.args.executor,
-                                      executor_instance=self.args.executor_instance,
+                                      executant_name=self.args.executant_name,
                                       config_file=self.args.config_file)
 
     @staticmethod
     @command_run_in_out
     def run_with_args(work_dir: str,
+                      asset_cache_dir: Optional[str],
                       model_upload_location: str,
                       pretrained_model_hash: str,
                       executor: str,
-                      executor_instance: str,
+                      executant_name: str,
                       src_revs: str,
                       dst_rev: str,
                       config_file: Optional[str],
                       tensorboard_dir: str,
                       mir_root: str = '.',
                       media_location: str = '') -> int:
-
         if not model_upload_location:
             logging.error("empty --model-location, abort")
             return MirCode.RC_CMD_INVALID_ARGS
-        if not src_revs:
-            logging.error('empty --src-revs, abort')
-            return MirCode.RC_CMD_INVALID_ARGS
-        src_typ_rev_tid = revs_parser.parse_single_arg_rev(src_revs)
-        if checker.check_src_revs(src_typ_rev_tid) != MirCode.RC_OK:
-            return MirCode.RC_CMD_INVALID_ARGS
-        dst_typ_rev_tid = revs_parser.parse_single_arg_rev(dst_rev)
-        if checker.check_dst_rev(dst_typ_rev_tid) != MirCode.RC_OK:
-            return MirCode.RC_CMD_INVALID_ARGS
+        src_typ_rev_tid = revs_parser.parse_single_arg_rev(src_revs, need_tid=False)
+        dst_typ_rev_tid = revs_parser.parse_single_arg_rev(dst_rev, need_tid=True)
         if not work_dir:
-            logging.error("empty work_dir, abort")
-            return MirCode.RC_CMD_INVALID_ARGS
-        if not config_file:
-            logging.warning('empty --config-file, abort')
-            return MirCode.RC_CMD_INVALID_ARGS
-        if not os.path.isfile(config_file):
-            logging.error(f"invalid --config-file {config_file}, not a file, abort")
-            return MirCode.RC_CMD_INVALID_ARGS
-
-        with open(config_file, "r") as f:
-            config = yaml.safe_load(f)
-        if 'class_names' not in config:
-            logging.error(f"no class_names in config file: {config_file}, abort")
-            return MirCode.RC_CMD_INVALID_ARGS
-
-        class_names = config['class_names']
-        if not class_names:
-            logging.error(f"empty class_names in config file: {config_file}, abort")
-            return MirCode.RC_CMD_INVALID_ARGS
-        if len(set(class_names)) != len(class_names):
-            logging.error(f"dumplicate class names in class_names: {class_names}, abort")
-            return MirCode.RC_CMD_INVALID_ARGS
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS, error_message='empty work_dir')
+        if not config_file or not os.path.isfile(config_file):
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
+                                  error_message=f"invalid --task-config-file: {config_file}")
+        if asset_cache_dir and not os.path.isabs(asset_cache_dir):
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
+                                  error_message=f"invalid --cache {config_file}, not an absolute path for directory")
 
         return_code = checker.check(mir_root,
                                     [checker.Prerequisites.IS_INSIDE_MIR_REPO, checker.Prerequisites.HAVE_LABELS])
         if return_code != MirCode.RC_OK:
             return return_code
 
+        with open(config_file, "r") as f:
+            config = yaml.safe_load(f)
+
+        task_parameters = _get_task_parameters(config)
+        if not isinstance(task_parameters, str):
+            raise MirRuntimeError(
+                error_code=MirCode.RC_CMD_INVALID_ARGS,
+                error_message=f"invalid {mir_settings.TASK_CONTEXT_PARAMETERS_KEY} in config: {config}")
+        if mir_settings.EXECUTOR_CONFIG_KEY not in config:
+            raise MirRuntimeError(
+                error_code=MirCode.RC_CMD_INVALID_ARGS,
+                error_message=f"invalid config file: {config_file}, needs: {mir_settings.EXECUTOR_CONFIG_KEY}")
+
+        executor_config = config[mir_settings.EXECUTOR_CONFIG_KEY]
+
+        class_names = executor_config.get('class_names', [])
+        if not class_names:
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
+                                  error_message=f"no class_names in config file: {config_file}")
+        if len(set(class_names)) != len(class_names):
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS,
+                                  error_message=f"dumplicate class names in class_names: {class_names}")
+
         task_id = dst_typ_rev_tid.tid
-        if not executor_instance:
-            executor_instance = f"default-training-{task_id}"
+        if not executant_name:
+            executant_name = f"default-training-{task_id}"
         if not tensorboard_dir:
             tensorboard_dir = os.path.join(work_dir, 'out', 'tensorboard')
+        asset_dir = asset_cache_dir or os.path.join(work_dir, 'in', 'assets')
 
         # if have model_hash, export model
         pretrained_model_names = _prepare_pretrained_models(model_location=model_upload_location,
@@ -293,45 +238,45 @@ class CmdTrain(base.BaseCommand):
                                                             dst_model_dir=os.path.join(work_dir, 'in', 'models'),
                                                             class_names=class_names)
 
-        # get train_ids, val_ids, test_ids
+        # get train_ids and val_ids
         train_ids = set()  # type: Set[str]
         val_ids = set()  # type: Set[str]
-        test_ids = set()  # type: Set[str]
         unused_ids = set()  # type: Set[str]
-        mir_datas = mir_storage_ops.MirStorageOps.load(mir_root=mir_root,
-                                                       mir_branch=src_typ_rev_tid.rev,
-                                                       mir_task_id=src_typ_rev_tid.tid,
-                                                       mir_storages=[mirpb.MirStorage.MIR_METADATAS])
-        mir_metadatas: mirpb.MirMetadatas = mir_datas[mirpb.MirStorage.MIR_METADATAS]
+        mir_metadatas: mirpb.MirMetadatas = mir_storage_ops.MirStorageOps.load_single_storage(
+            mir_root=mir_root,
+            mir_branch=src_typ_rev_tid.rev,
+            mir_task_id=src_typ_rev_tid.tid,
+            ms=mirpb.MirStorage.MIR_METADATAS)
         for asset_id, asset_attr in mir_metadatas.attributes.items():
             if asset_attr.tvt_type == mirpb.TvtTypeTraining:
                 train_ids.add(asset_id)
             elif asset_attr.tvt_type == mirpb.TvtTypeValidation:
                 val_ids.add(asset_id)
-            elif asset_attr.tvt_type == mirpb.TvtTypeTest:
-                test_ids.add(asset_id)
             else:
                 unused_ids.add(asset_id)
         if not train_ids:
-            logging.error("no training set; abort")
-            return MirCode.RC_CMD_INVALID_ARGS
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS, error_message='no training set')
         if not val_ids:
-            logging.error("no validation set; abort")
-            return MirCode.RC_CMD_INVALID_ARGS
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS, error_message='no validation set')
 
         if not unused_ids:
-            logging.info(f"training: {len(train_ids)}, validation: {len(val_ids)}, test: {len(test_ids)}")
+            logging.info(f"training: {len(train_ids)}, validation: {len(val_ids)}")
         else:
-            logging.warning(f"training: {len(train_ids)}, validation: {len(val_ids)}, test: {len(test_ids)}, "
-                            f"unused: {len(unused_ids)}")
+            logging.warning(f"training: {len(train_ids)}, validation: {len(val_ids)}" f"unused: {len(unused_ids)}")
 
         # export
         logging.info("exporting assets")
+
         os.makedirs(work_dir, exist_ok=True)
+
         work_dir_in = os.path.join(work_dir, "in")
+        work_dir_annotations = os.path.join(work_dir_in, 'annotations')
+        os.makedirs(work_dir_annotations, exist_ok=True)
+
         work_dir_out = os.path.join(work_dir, "out")
-        os.makedirs(work_dir_in, exist_ok=True)
         os.makedirs(work_dir_out, exist_ok=True)
+
+        os.makedirs(asset_dir, exist_ok=True)
         os.makedirs(tensorboard_dir, exist_ok=True)
 
         # type names to type ids
@@ -341,87 +286,69 @@ class CmdTrain(base.BaseCommand):
         if not type_ids_list:
             logging.info(f"type ids empty, please check config file: {config_file}")
             return MirCode.RC_CMD_INVALID_ARGS
+
+        if not context.check_class_ids(mir_root=mir_root, current_class_ids=type_ids_list):
+            raise MirRuntimeError(error_code=MirCode.RC_CMD_INVALID_ARGS, error_message='project class ids mismatch')
+
         type_id_idx_mapping = {type_id: index for (index, type_id) in enumerate(type_ids_list)}
 
         # export train set
-        work_dir_in_train = os.path.join(work_dir_in, 'train')
-        if os.path.isdir(work_dir_in_train):
-            shutil.rmtree(work_dir_in_train)
         data_exporter.export(mir_root=mir_root,
                              assets_location=media_location,
                              class_type_ids=type_id_idx_mapping,
                              asset_ids=train_ids,
-                             asset_dir=work_dir_in_train,
-                             annotation_dir=work_dir_in_train,
+                             asset_dir=asset_dir,
+                             annotation_dir=work_dir_annotations,
                              need_ext=True,
-                             need_id_sub_folder=False,
+                             need_id_sub_folder=True,
                              base_branch=src_typ_rev_tid.rev,
                              base_task_id=src_typ_rev_tid.tid,
                              format_type=data_exporter.ExportFormat.EXPORT_FORMAT_ARK,
-                             index_file_path=os.path.join(work_dir_in_train, 'index.tsv'),
-                             index_prefix='/in/train')
+                             index_file_path=os.path.join(work_dir_in, 'train-index.tsv'),
+                             index_assets_prefix='/in/assets',
+                             index_annotations_prefix='/in/annotations')
 
         # export validation set
-        work_dir_in_val = os.path.join(work_dir_in, 'val')
-        if os.path.isdir(work_dir_in_val):
-            shutil.rmtree(work_dir_in_val)
         data_exporter.export(mir_root=mir_root,
                              assets_location=media_location,
                              class_type_ids=type_id_idx_mapping,
                              asset_ids=val_ids,
-                             asset_dir=work_dir_in_val,
-                             annotation_dir=work_dir_in_val,
+                             asset_dir=asset_dir,
+                             annotation_dir=work_dir_annotations,
                              need_ext=True,
-                             need_id_sub_folder=False,
+                             need_id_sub_folder=True,
                              base_branch=src_typ_rev_tid.rev,
                              base_task_id=src_typ_rev_tid.tid,
                              format_type=data_exporter.ExportFormat.EXPORT_FORMAT_ARK,
-                             index_file_path=os.path.join(work_dir_in_val, 'index.tsv'),
-                             index_prefix='/in/val')
-
-        # export test set (if we have)
-        if test_ids:
-            work_dir_in_test = os.path.join(work_dir_in, 'test')
-            if os.path.isdir(work_dir_in_test):
-                shutil.rmtree(work_dir_in_test)
-            data_exporter.export(mir_root=mir_root,
-                                 assets_location=media_location,
-                                 class_type_ids=type_id_idx_mapping,
-                                 asset_ids=test_ids,
-                                 asset_dir=work_dir_in_test,
-                                 annotation_dir=work_dir_in_test,
-                                 need_ext=True,
-                                 need_id_sub_folder=False,
-                                 base_branch=src_typ_rev_tid.rev,
-                                 base_task_id=src_typ_rev_tid.tid,
-                                 format_type=data_exporter.ExportFormat.EXPORT_FORMAT_ARK,
-                                 index_file_path=os.path.join(work_dir_in_test, 'index.tsv'),
-                                 index_prefix='/in/test')
+                             index_file_path=os.path.join(work_dir_in, 'val-index.tsv'),
+                             index_assets_prefix='/in/assets',
+                             index_annotations_prefix='/in/annotations')
 
         logging.info("starting train docker container")
 
+        available_gpu_id = config.get(mir_settings.TASK_CONTEXT_KEY, {}).get('available_gpu_id', '')
+
         # generate configs
         out_config_path = os.path.join(work_dir_in, "config.yaml")
         executor_config = _generate_config(
-            config=config,
+            executor_config=executor_config,
             out_config_path=out_config_path,
             task_id=task_id,
             pretrained_model_params=[os.path.join('/in/models', name) for name in pretrained_model_names])
 
-        gpu_id = executor_config['gpu_id']
-
         # start train docker and wait
         path_binds = []
-        path_binds.append(f"-v{work_dir_in}:/in")
+        path_binds.append(f"-v{work_dir_in}:/in")  # annotations, models, train-index.tsv, val-index.tsv, config.yaml
+        path_binds.append(f"-v{asset_dir}:/in/assets:ro")  # assets
         path_binds.append(f"-v{work_dir_out}:/out")
         path_binds.append(f"-v{tensorboard_dir}:/out/tensorboard")
-        shm_size = _get_shm_size(config_file)
 
-        cmd = ['nvidia-docker', 'run', '--rm', f"--shm-size={shm_size}"]
+        cmd = ['nvidia-docker', 'run', '--rm', f"--shm-size={_get_shm_size(executor_config=executor_config)}"]
         cmd.extend(path_binds)
-        cmd.extend(['--gpus', f"\"device={gpu_id}\""])
-        cmd.extend(['--user', f"{os.getuid()}:{os.getgid()}"])
-        cmd.extend(['--name', f"{executor_instance}"])
+        if available_gpu_id:
+            cmd.extend(['--gpus', f"\"device={available_gpu_id}\""])
+        cmd.extend(['--user', f"{os.getuid()}:{os.getgid()}"])  # run as current user
+        cmd.extend(['--name', f"{executant_name}"])  # executor name used to stop executor
         cmd.append(executor)
 
         task_code = MirCode.RC_OK
@@ -434,46 +361,51 @@ class CmdTrain(base.BaseCommand):
             task_code = MirCode.RC_CMD_CONTAINER_ERROR
             task_error_msg = str(e)
 
+        # gen task_context
+        task_context = {
+            'src_revs': src_revs,
+            'dst_rev': dst_rev,
+            'executor': executor,
+            mir_settings.PRODUCER_KEY: mir_settings.PRODUCER_NAME,
+            mir_settings.TASK_CONTEXT_PARAMETERS_KEY: task_parameters
+        }
+
         # save model
         logging.info("saving models")
         model_sha1, model_mAP = _process_model_storage(out_root=work_dir_out,
                                                        model_upload_location=model_upload_location,
                                                        executor_config=executor_config,
-                                                       task_context={
-                                                           'src_revs': src_revs,
-                                                           'dst_rev': dst_rev,
-                                                           'executor': executor
-                                                       })
+                                                       task_context=task_context)
 
-        # update metadatas and task with finish state and model hash
-        mir_tasks = _update_mir_tasks(mir_root=mir_root,
-                                      src_rev_tid=src_typ_rev_tid,
-                                      dst_rev_tid=dst_typ_rev_tid,
-                                      model_sha1=model_sha1,
-                                      mAP=model_mAP,
-                                      task_ret_code=task_code,
-                                      task_err_msg=task_error_msg)
+        # commit task
+        task = mir_storage_ops.create_task(task_type=mirpb.TaskType.TaskTypeTraining,
+                                           task_id=dst_typ_rev_tid.tid,
+                                           message='training',
+                                           model_mAP=model_mAP,
+                                           model_hash=model_sha1,
+                                           return_code=task_code,
+                                           return_msg=task_error_msg,
+                                           serialized_task_parameters=task_parameters,
+                                           serialized_executor_config=yaml.safe_dump(executor_config),
+                                           executor=executor,
+                                           src_revs=src_revs,
+                                           dst_rev=dst_rev)
 
         if task_code != MirCode.RC_OK:
-            raise MirRuntimeError(error_code=task_code,
-                                  error_message=task_error_msg,
-                                  needs_new_commit=True,
-                                  mir_tasks=mir_tasks)
+            raise MirRuntimeError(error_code=task_code, error_message=task_error_msg, needs_new_commit=True, task=task)
 
         mir_storage_ops.MirStorageOps.save_and_commit(mir_root=mir_root,
                                                       mir_branch=dst_typ_rev_tid.rev,
-                                                      task_id=dst_typ_rev_tid.tid,
                                                       his_branch=src_typ_rev_tid.rev,
-                                                      mir_datas={mirpb.MirStorage.MIR_TASKS: mir_tasks},
-                                                      commit_message=dst_typ_rev_tid.tid)
+                                                      mir_datas={},
+                                                      task=task)
 
         logging.info("training done")
 
         return MirCode.RC_OK
 
 
-def bind_to_subparsers(subparsers: argparse._SubParsersAction,
-                       parent_parser: argparse.ArgumentParser) -> None:
+def bind_to_subparsers(subparsers: argparse._SubParsersAction, parent_parser: argparse.ArgumentParser) -> None:
     train_arg_parser = subparsers.add_parser("train",
                                              parents=[parent_parser],
                                              description="use this command to train current workspace",
@@ -494,14 +426,19 @@ def bind_to_subparsers(subparsers: argparse._SubParsersAction,
                                   required=False,
                                   help='model hash to be used')
     train_arg_parser.add_argument("-w", required=True, dest="work_dir", type=str, help="work place for training")
+    train_arg_parser.add_argument('--asset-cache-dir',
+                                  required=False,
+                                  dest='asset_cache_dir',
+                                  type=str,
+                                  help='asset cache directory')
     train_arg_parser.add_argument("--executor",
                                   required=True,
                                   dest="executor",
                                   type=str,
                                   help="docker image name for training")
-    train_arg_parser.add_argument('--executor-instance',
+    train_arg_parser.add_argument('--executant-name',
                                   required=False,
-                                  dest='executor_instance',
+                                  dest='executant_name',
                                   type=str,
                                   help='docker container name for training')
     train_arg_parser.add_argument("--src-revs",
@@ -514,12 +451,12 @@ def bind_to_subparsers(subparsers: argparse._SubParsersAction,
                                   type=str,
                                   required=True,
                                   help="rev@tid: destination branch name and task id")
-    train_arg_parser.add_argument("--config-file",
+    train_arg_parser.add_argument("--task-config-file",
                                   dest="config_file",
                                   type=str,
                                   required=True,
                                   help="path to executor config file")
-    train_arg_parser.add_argument("--tensorboard",
+    train_arg_parser.add_argument("--tensorboard-dir",
                                   dest="tensorboard_dir",
                                   type=str,
                                   required=False,
